/****************************************************************************
*
* File:             ConnectionManager.c
* Author:           Nikolaos Radouniklis
* Date:             07/09/96
* Last changed:     07/15/96
* Description:      Connection Manager for the entire CME session
*                   Provides functionality for following tasks:
*                   1) Invitation
*                   2) DisConnection
*
* Comment:			
*
****************************************************************************/


/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <msql.h>
#include <tcl.h>
#include <tk.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"
#include "Database.h"
#include "TclInterp.h"


/* Global variables */
extern int G_Sock; /*global socket descriptor, necessary due to Tk_FileEventHandler */
extern InvitationLetterStruct InvitationLetter; /* global invitation letter */
extern Tcl_Interp *G_interp;

/* prototypes */
int MakeSocket (unsigned short int port);
int InitSocketAddress (struct sockaddr_in *name, const char *hostname, unsigned short int port);
int CheckParticipant ( char* ParticipantHost, int Port );
int InviteParticipant ( char* Hostname, int Port, InvitationLetterStruct *InvitationLetter );
int SendInvitationLetter ( int sockd, InvitationLetterStruct *InvitationLetter );
int ReadInvitationLetter ( int sockd, InvitationLetterStruct *InvitationLetter );
int WaitForInvitation (int port);
void ProcessInvitation (ClientData clientdata, int mask);



/****************************************************************************
*
* Procedure:        MakeSocket
* Param:			[in] port: Portnumber for socket
* Result:           sock: Socket descriptor
* Date:             07/09/96
* Last changed:     07/09/96
* Description:		Creates a socket descriptor and binds it to
*					port (basic functions used: socket, bind)
* 
*
****************************************************************************/
int MakeSocket (unsigned short int port)
{

	int sock; 	/* socket descriptor to be returned */
  	struct sockaddr_in name;

  	/* Create the socket descriptor */
  	sock = socket (PF_INET, SOCK_STREAM, 0);
  	if (sock < 0)
   	{
   		perror ("MakeSocket: SOCKET");
   		exit (EXIT_FAILURE);
   	}

  	/* Fill the socket data structure */
  	name.sin_family = AF_INET;
  	name.sin_port = htons (port);
  	name.sin_addr.s_addr = htonl (INADDR_ANY);
  	if (bind (sock, (struct sockaddr *) &name, sizeof (name)) < 0)
    {
      	perror ("MakeSocket: BIND");
      	exit (EXIT_FAILURE);
    }

 	/* Return the socket descriptor */ 
	return sock;
}




/**************************************************************************** 
*
* Procedure:        InitSocketAddress
* Param:            [out] name: Socket address to be initialized
*					[in]  hostname: (e.g icsib8.icsi.berkeley.edu)
*					[in]  port: Portnumber to be assigned to name
* Result:           (-1) OK; (0) Fail
* Date:             07/09/96
* Last changed:     07/15/96 
* Description:      Initializes the sockaddr_in structure with the 
*					necessary host&port information
* 
****************************************************************************/ 
int InitSocketAddress (struct sockaddr_in *name, const char *hostname, unsigned short int port)
{

	struct hostent *hostinfo;

	/* fill the data structure with host & port information */
  	name->sin_family = AF_INET;
  	name->sin_port = htons (port);
  	hostinfo = gethostbyname (hostname);
  	if (hostinfo == NULL)
    {
      	fprintf (stderr, "InitSocketAddress: Unknown host %s.\n", hostname);
      	return (0);
    }
  	name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
	return (-1);
}




/****************************************************************************
*
* Procedure:        CheckParticipant
* Param:            [in]  ParticipantHost: (e.g icsib8.icsi.berkeley.edu)
*                   [in]  Port: Portnumber where participant is listening to
*                   [in]  InvitationLetter: (structure with invitation info)
* Result:           CONNECT_OK, CONNECT_FAIL
* Date:             07/09/96
* Last changed:     07/09/96
* Description:		Check if Participant server process is running
*
****************************************************************************/
int CheckParticipant ( char* ParticipantHost, int Port )
{

    int sockd; /* Socket descriptor */
    struct sockaddr_in servername;
 
    /* Create the socket.   */
    sockd = socket (PF_INET, SOCK_STREAM, 0);
    if (sockd < 0)
    {
        perror ("InviteParticipant: SOCKET ");
        exit (EXIT_FAILURE);
    }
 
    /* Connect to the server.   */
    if ( InitSocketAddress (&servername, ParticipantHost, Port) == 0 )
	{
		/* Wrong participant address */
		return CONNECT_FAIL;
	}
 
    /* Connect to ParticipantHost (server) 	*/
	/* Check if participant is ONLINE		*/	
    if (0 > connect (sockd, (struct sockaddr *) &servername, sizeof (servername)))
    {
        perror ("InviteParticipant: CONNECT ");
        return CONNECT_FAIL;
    }

    /* Delete the socket descriptor */
    close (sockd);
 
    /*Return successfullly */
    return CONNECT_OK;

}



/****************************************************************************  
*
* Procedure:        InviteParticipant
* Param:            [in]  ParticipantHost: (e.g icsib8.icsi.berkeley.edu)
*                   [in]  Port: Portnumber where participant is listening to
*					[in]  InvitationLetter: (structure with invitation info)
* Result:           (-1) OK; 0 ERROR
* Date:             07/09/96  
* Last changed:     07/09/96  
* Description:      
*                   
****************************************************************************/
int InviteParticipant ( char* ParticipantHost, int Port, InvitationLetterStruct *InvitationLetter )
{

 
	int sockd; /* Socket descriptor */
  	struct sockaddr_in servername;

  	/* Create the socket.   */
  	sockd = socket (PF_INET, SOCK_STREAM, 0);
  	if (sockd < 0)
    {
      	perror ("InviteParticipant: SOCKET ");
      	exit (EXIT_FAILURE);
    }

  	/* Connect to the server.   */
  	InitSocketAddress (&servername, ParticipantHost, Port);

	/* Connect to ParticipantHost (server) */
  	if (0 > connect (sockd, (struct sockaddr *) &servername, sizeof (servername)))
    {
      	perror ("InviteParticipant: CONNECT ");
      	exit (EXIT_FAILURE);
    }
  
	/* Send  InvitationLetter to the ParicipantHost   */
  	SendInvitationLetter ( sockd, InvitationLetter );

	/* Delete the socket descriptor */
  	close (sockd);

	/*Return successfullly */
	return (-1);

}






/****************************************************************************
*
* Procedure:        SendInvitationLetter
* Param:            [in]  sockd: (Socket descriptor to participant)
*                   [in]  InvitationLetter: (structure with invitation info)
* Result:
* Date:             07/09/96
* Last changed:     07/09/96  
* Description:      Send the invitation to socket descriptor 
*					which connects the session initiator to the participant
*   
****************************************************************************/
int SendInvitationLetter ( int sockd, InvitationLetterStruct *InvitationLetter )
{

	/* How many Bytes are written? */
	int NBytes;
   
	NBytes = write (sockd, InvitationLetter, sizeof(*InvitationLetter) );
  	if (NBytes < 0)
    {
      perror ("SendInvitationLetter: WRITE");
      exit (EXIT_FAILURE);
	}

	return (-1);	
}





/**************************************************************************** 
* 
* Procedure:        ReadInvitationLetter
* Param:            [in]  sockd: (Socket descriptor to participant)
*                   [in]  InvitationLetter: (structure with invitation info)
* Result: 			(0) EOF is reached; (-1) Data read
* Date:             07/09/96 
* Last changed:     07/09/96   
* Description:      Receive the invitation from the session initiator 
*   
****************************************************************************/
int ReadInvitationLetter ( int sockd, InvitationLetterStruct *InvitationLetter )
{

    /* How many Bytes are read? */
    int NBytes;
 
    NBytes = read (sockd, InvitationLetter, sizeof(*InvitationLetter) );
    if (NBytes < 0)
    {
      	perror ("ReadInvitationLetter: READ");
      	exit (EXIT_FAILURE);
    }
	else if (NBytes == 0)
	{
		/* end of file (EOF) reached */
		return (0);	
	}
		
	/* Data read */
	return (-1);
}



 
/****************************************************************************
*
* Procedure:        WaitForInvitation
* Param:            [in]  port: Portnumber to listen
*                   
* Result:           (-1) OK, 0 ERROR
* Date:             07/09/96
* Last changed:     07/09/96
* Description:      Wait for the InvitationLetter
*					Set the global Socket Descriptor G_Sock
*					before invoking the File Event Handler
*
****************************************************************************/
int WaitForInvitation (int port)
{

  	int sockd;

  	/* Create the socket and set it up to accept connections.  */
  	sockd = MakeSocket (port);

	/* Listen for incoming requests */
  	if (listen (sockd, 5) < 0)
  	{
  		perror ("WaitForInvitation: LISTEN");
  		exit (EXIT_FAILURE);
  	}
	/* set the global socket descriptor G_Sock for ProcessInvitation */
	G_Sock = sockd;
 
  	/* Create now the TK File Event Handler to wait for incoming calls */
	Tk_CreateFileHandler (sockd, TK_READABLE, ProcessInvitation, (ClientData) NULL);

	return (-1);
}





/****************************************************************************
*
* Procedure:        ProcessInvitation
* Param:            [in] clientdata
*					[in] mask
* Result:           
* Date:             07/10/96
* Last changed:     07/15/96
* Description:      Read the incoming invitation request
*					!Global variable G_Sock
*					!Global structure InvitationLetter
*
****************************************************************************/
void ProcessInvitation (ClientData clientdata, int mask)
{

	int newsock; /* socket descriptor to read data */
	char MyUserID[CM_MAXUSRLEN]; /* String to hold User ID */

	
	/* get the User-Id of the active process */
	cuserid ( MyUserID );

	/* Create new socket descriptor to read data */	
	newsock = accept (G_Sock, (struct sockaddr *) NULL, (int *) NULL );
	if ( newsock == -1 ) 
	{
		perror ("ProcessInvitation: ACCEPT");
		exit (1);
	}

	
	/* Read now the data through socket descriptor newsock (ignore previous EOFs)*/
	if (ReadInvitationLetter ( newsock, &InvitationLetter ) != 0)
	{
		/* go to session management */
    	Tcl_Eval (G_interp, "build_session");
	}


}

