/****************************************************************************
*
* File:             Database.c
* Author:           Nikolaos Radouniklis
* Date:             06/01/96
* Last changed:     07/18/96
* Description:      database API functions for CME-DB
*
*
****************************************************************************/

/* system includes */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <msql.h>

/* my includes */
#include "Constants.h"

/* prototypes */
#include "Database.h"


/* defines */


/****************************************************************************
*
* Function:         DBConnect
* Param:            [in] DBName
* Returns:          [res] sock (Socket descriptor)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Connects to the CME database
*
****************************************************************************/



int DBConnect (char *DBName)
{

    int sock, retval;


    /********************************/
    /* look for local msqld         */
    /********************************/

    sock = msqlConnect (NULL);
    if (sock == -1 )
    {
        printf (" err(DBConnect): Failure of msqlConnect \n");
        exit (8);
    }

    /********************************/
    /* connect to CME database      */
    /********************************/

    retval = msqlSelectDB ( sock, DBGetName() );
    if (retval == -1)
    {
        printf ("err(DBConnect): Can't connect to CME_DB\n");
        exit (8);
    }
 
    return sock;
 
}





/****************************************************************************
*  
* Function:         DBRelease 
* Param:            [in] sock (Socket connection of database)
* Returns:          -1 (OK); 0 (Error)
* Date:             06/01/96 
* Last changed:     06/01/96 
* Description:      Connects to the CME database
*  
****************************************************************************/

int DBRelease (int sock)
{

    msqlClose (sock);
	return (-1);

}






/****************************************************************************
*
* Procedure:        error_handler
* Date:             05/30/96
* Last changed:     06/02/96
* Description:      Handle error in intialisation
* Parameters:
*
****************************************************************************/

int error_handler ()
{

    printf ("ERROR: %s \n", msqlErrMsg );
	return (-1);
}



/****************************************************************************
*
* Function:         DBClearTable
* Param:            [in] sock (Socket connection of database)
*					[in] table (Name of the table to be cleared)
* Returns:          -1 (OK); 0 (Error)
* Date:             06/04/96
* Last changed:     06/04/96
* Description:      Connects to the CME database
*
****************************************************************************/
 
int DBClearTable (int socket, char *Table)
{
	
    char SQL_statement[SQL_STATEMENT_MAX];
            
	sprintf ( SQL_statement, "  DELETE FROM %s",\
                                Table);
    if (msqlQuery (socket, SQL_statement) == -1)
   		error_handler();
	
	return (-1); 
} 



/****************************************************************************
*
* Function:         DBAverage
* Param:            [in] sock (Socket connection to database)
*                   [in] table (Name of the table)
*					[in] attr  (Name of the attribute)
*					[in] samples (no of samples to be considered) 
*					[in] Condition (additional condition for SQL-Query) or NULL
*					[out] Avg (average value of attr over samples)
* Returns:         	(-1) OK; (0) Error; 
* Date:             06/07/96
* Last changed:     06/26/96
* Description:      Computes the average value of the attribute <attr>
*					in the table <table>
*
****************************************************************************/

int DBAverage (int sock, char *Table, char *Attr, int Samples, char* Condition, float *Avg)
{

	static char SQL_statement[SQL_STATEMENT_MAX];
	static int  max, min;
	float Sum=0.0; 
	m_result    *db_result;
    m_row       db_row;


	/* get first the last entered value of attr */
	/* which corresponds to the highest counter */
	max = DBGetLastRow ( sock, Table, "A_Counter") - 1;
	min = max - Samples + 1;
	/* compute now the average value */
	sprintf ( SQL_statement, "	SELECT %s FROM %s \
								WHERE A_Counter >= %d \
								AND A_Counter <= %d",\
								Attr, Table, min, max);

	/* add Condition if not NULL to SQL statement */
	if (Condition != NULL)
	{
		strcat (SQL_statement, " AND ");
		strcat (SQL_statement, Condition);
	}

	if (msqlQuery (sock, SQL_statement) == -1)
   		error_handler();
    db_result = msqlStoreResult();

    while ( (db_row=msqlFetchRow(db_result)) != NULL )
	{
		Sum = Sum + atof (db_row[0]);
	}
	*Avg = Sum / Samples; 
	msqlFreeResult(db_result);
	return (-1);

}



/****************************************************************************
*
* Function:         DBGetAttr
* Param:            [in] sock (Socket connection to database)
*                   [in] table (Name of the table)
*                   [in] attr  (Name of the attribute)
*                   [in] Condition (additional condition for SQL-Query) or NULL
*                   [out] Value (string value); '\0' if no value exists;
* Returns:          (-1) OK; (0) Error;  
* Date:             06/26/96   
* Last changed:     06/26/96
* Description:      retrieve the attribute value <Value> of <Attr>
*                   
*
****************************************************************************/
 
int DBGetAttr (int sock, char *Table, char *Attr, char *Condition, char *Value)
{
 
    static char SQL_statement[SQL_STATEMENT_MAX];
	static int max;
    m_result    *db_result;
    m_row       db_row;   
 
 
    /* get first the last entered value of attr */
    /* which corresponds to the highest counter */
    max = DBGetLastRow ( sock, Table, "A_Counter") - 1;

    /* generate now the SQL statement */  
    sprintf ( SQL_statement, "  SELECT %s FROM %s \
                                WHERE A_Counter = %d",\
                                Attr, Table, max);
 
    /* add Condition if not NULL to SQL statement */
    if (Condition != NULL) 
    {
        strcat (SQL_statement, " AND ");
        strcat (SQL_statement, Condition);
    }
    
    if (msqlQuery (sock, SQL_statement) == -1)
        error_handler();
   
	/* fill return value: [out] Value */ 
	db_result = msqlStoreResult();
    db_row=msqlFetchRow(db_result);

	if (db_row != NULL)
		strcpy (Value, db_row[0]);
	else
		Value[0] = '\0';
    
    msqlFreeResult(db_result);
   
	return (-1);
 
}




/****************************************************************************
*
* Function:         DBGetLastRow
* Param:            [in] sock (Socket connection to database)
*                   [in] Table (Name of the table)
*                   [in] attr  (Name of the attribute)
* Returns:          Last (Counter Value of last inserted row)
* Date:             06/06/96
* Last changed:     06/06/96
* Description:      (--> Returns)
*                   
****************************************************************************/

int	DBGetLastRow (int sock, char *Table, char *Attr)
{

	static char SQL_statement[SQL_STATEMENT_MAX];
	m_result	*db_result;
	m_row		db_row;
	int Last;

	sprintf (SQL_statement, "	SELECT %s FROM %s \
								ORDER BY %s DESC LIMIT 1",
								Attr, Table, Attr);
	if (msqlQuery (sock, SQL_statement) == -1)
        error_handler();

	db_result = msqlStoreResult();
	if ((db_row = msqlFetchRow(db_result)) != NULL)
		Last = atoi (db_row[0]);
	else
		Last = 0;

	msqlFreeResult(db_result);
	return (Last);


}



/****************************************************************************
*
* Function:         DBFieldIsType
* Param:            [in] sock (Socket connection to database)
*                   [in] Table (Name of the table)
*                   [in] attr  (Name of the attribute)
* Returns:          INT_TYPE, CHAR_TYPE, REAL_TYPE (type of attribute)
* Date:             06/07/96
* Last changed:     06/07/96
* Description:      Retrieves the attribute type of attr in Table
*
****************************************************************************/

int DBFieldIsType (int sock, char *Table, char *Attr)
{

	m_result	*db_result;
	m_field		*db_field;
	int			retval;
	
	db_result = msqlListFields (sock, Table);
	do
	{
		db_field = msqlFetchField (db_result);
		/* if fieldname matches return the fieldtype */
		if (strcmp(Attr, (*db_field).name) == 0 )
		{
			retval = (*db_field).type;
			msqlFreeResult(db_result);
			return ( retval);
		}
	} while (db_field != NULL);

	/* at this point there was no match */
	msqlFreeResult(db_result);
	return (0);
}
	
	



/****************************************************************************
*
* Procedure:        drop_tables (option -d)
* Date:             05/29/96
* Last changed:     07/22/96
* Description:      drop the CME database tables
* Parameters:    
*
****************************************************************************/

int drop_tables()
{
 
    int sock, retval;
 
    /********************************/
    /* look for msqld               */
    /********************************/
 
    sock = msqlConnect (NULL);
    if (sock == -1 )
    {
        printf (" err: Failure of msqlConnect \n");
        exit (8);
    }
 
    /********************************/
    /* connect to CME database      */
    /********************************/
 
    retval = msqlSelectDB ( sock, DBGetName() );
    if (retval == -1)
    {
        printf ("err: Can't connect to DATABASE\n");
        exit (8);
    }
 
    /********************************/
    /* drop tables                	*/
    /********************************/
    
    if (msqlQuery (sock, "DROP TABLE T_ActiveMS") == -1 )
		error_handler();
	printf ("> Table T_ActiveMS dropped \n");
    if (msqlQuery (sock, "DROP TABLE T_ActiveMSM" ) == -1 )
		error_handler();
	printf ("> Table T_ActiveMSM dropped \n");
	if (msqlQuery (sock, "DROP TABLE T_VicInfo") == -1 )
		error_handler();
	printf ("> Table T_VicInfo dropped \n");
	if (msqlQuery (sock, "DROP TABLE T_VatInfo") == -1 )
		error_handler();
	printf ("> Table T_VatInfo dropped \n");
	if (msqlQuery (sock, "DROP TABLE T_Wish") == -1 )
		error_handler();
	printf ("> Table T_Wish dropped \n");
	
 
    msqlClose (sock);

	return (-1);
 
}






/****************************************************************************
*
* Procedure:        build_tables 
* Date:             05/29/96
* Last changed:     07/22/96
* Description:      build the CME database tables
* Parameters:       
*
****************************************************************************/

int build_tables ()
{

	int sock, retval;


	/********************************/
	/* look for msqld 				*/
	/********************************/

	sock = msqlConnect (NULL);
	if (sock == -1 )
	{
		printf (" err: Failure of msqlConnect \n");
		exit (8);
	}

	/********************************/
	/* connect to CME database		*/
	/********************************/

	retval = msqlSelectDB ( sock, DBGetName() ); 
	if (retval == -1)
	{
		printf ("err: Can't connect to DATABASE\n");
		exit (8);
	}

	/********************************/
	/* create tables 				*/
	/********************************/

	/* T_ActiveMS */	
	if (msqlQuery (sock, "CREATE TABLE T_ActiveMS \
					( A_Pid int primary key, A_MSName char(20) )" ) == -1)
		error_handler();
	printf ("> Table T_ActiveMS built \n");

	/* T_ActiveMSM */	
	if (msqlQuery (sock, "CREATE TABLE T_ActiveMSM \
					( A_Pid int primary key, A_MSName char(20) )" ) == -1)
		error_handler();
	printf ("> Table T_ActiveMSM built \n");
	

	/* T_VicInfo */
	if (msqlQuery (sock, "CREATE TABLE T_VicInfo\
                    (   A_Counter int ,\
						A_Participant char(25),\
                        A_Compression char(5),\
                        A_FrameRate real,\
                        A_Bandwith int,\
						A_Loss int )" ) == -1)
        error_handler();
    printf ("> Table T_VicInfo built \n");


    /* T_VatInfo */
    if (msqlQuery (sock, "CREATE TABLE T_VatInfo\
                    (   A_Counter int ,\
                        A_Participant char(25),\
                        A_Compression int  )" ) == -1)
        error_handler();
    printf ("> Table T_VatInfo built \n");



	/* T_Wish */
	if (msqlQuery (sock, "CREATE TABLE T_Wish\
					(	A_Participant 	char(25),\
						A_MSName		char(5),\
						A_QoS			int )" ) == -1)
		error_handler ();
	printf ("> Table T_Wish built \n");


	/***********************************/
	/* empty tables					   */		
	/***********************************/

    DBClearTable ( sock, "T_ActiveMS" );
    DBClearTable ( sock, "T_ActiveMSM" );
    DBClearTable ( sock, "T_VicInfo" );
    DBClearTable ( sock, "T_VatInfo" );
    DBClearTable ( sock, "T_Wish" );



	msqlClose (sock);


	return (-1);

}	

/****************************************************************************
*
* Procedure:        DBGetName
* Date:             07/18/96
* Last changed:     07/18/96
* Description:      concatenates CME_Prefix && Host --> DBName
* Parameters:		--
*
* Returns:			DBName 
*
****************************************************************************/

char *DBGetName ()
{
	static char Host  [MAX_CHARBUFFER];
	static char DBName[MAX_CHARBUFFER];

	gethostname ( Host, MAX_CHARBUFFER -1 );
	sprintf ( DBName, "%s%s", CME_PREFIX, Host );

	return ( DBName );

}

/****************************************************************************
*
* Function:         DBCreate
* Param:
* Returns:          (-1) OK; (0) Error
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Creates a CME database
*
****************************************************************************/
 
int DBCreate ()
{
    int sock;

    /* look for msqld               */
    sock = msqlConnect (NULL);
    if (sock == -1 )
    {
        printf (" err: Failure of msqlConnect \n");
        return (0);
    }

	/* Create Database */
	msqlCreateDB ( sock, DBGetName() );

	/* close DB */
	msqlClose ( sock );

	return (-1);
}


/****************************************************************************
*
* Function:         DBDestroy
* Param:
* Returns:          (-1) OK; (0) Error
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Destroys a CME database
*
****************************************************************************/
 
int DBDestroy ()
{
	int sock;
 
    /* look for msqld               */ 
    sock = msqlConnect (NULL); 
    if (sock == -1 ) 
    { 
        printf (" err: Failure of msqlConnect \n"); 
        return (0);
    } 
 
    /* Create Database */ 
    msqlDropDB ( sock, DBGetName() ); 
 
    /* close DB */ 
    msqlClose ( sock );
 
    return (-1); 
}




/****************************************************************************
*
* Function:         DBAvg
* Param:            [in] sock (Socket connection to database)
*                   [in] table (Name of the table)
*                   [in] attr  (Name of the attribute)
*                   [in] Condition (additional condition for SQL-Query) or NULL
*                   [out] Avg (average value of attr )
* Returns:          (-1) OK; (0) Error;
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      Computes the average value of the attribute <attr>
*                   in the table <table>
*
****************************************************************************/

int DBAvg (int sock, char *Table, char *Attr, char* Condition, float *Avg)
{

    static char SQL_statement[SQL_STATEMENT_MAX];
    float Sum=0.0;  
    m_result    *db_result;
    m_row       db_row;
	int i;


    /* compute now the average value */
    sprintf ( SQL_statement, "  SELECT %s FROM %s ",\
                                Attr, Table);

    /* add Condition if not NULL to SQL statement */
    if (Condition != NULL)
    {
        strcat (SQL_statement, " WHERE ");
        strcat (SQL_statement, Condition);
    }
 
    if (msqlQuery (sock, SQL_statement) == -1)
        error_handler();
    db_result = msqlStoreResult();

	i = 0; 
    while ( (db_row=msqlFetchRow(db_result)) != NULL )
	{
        Sum = Sum + atof (db_row[0]);
		i+=1;
	}

	if (i != 0 )
    	*Avg = Sum / i;
	else
		*Avg = QOS_NO_VALUE;


    msqlFreeResult(db_result);
    return (-1);
 
}


/****************************************************************************
*
* Function:         DBRowsExist
* Param:            [in] sock (Socket connection to database)
*                   [in] SQLstatement
* Returns:          NumRows ( number of rows for SQL statement)
* Date:             07/25/96
* Last changed:     07/25/96
* Description:      Checks if rows exist for a given SQLstatement
*                   
*
****************************************************************************/

int DBRowsExist (int sock, char *SQL_statement )
{

    m_result    *db_result;
    int			NumRows; 


    if (msqlQuery (sock, SQL_statement) == -1)
        error_handler();
    db_result = msqlStoreResult();

	NumRows = msqlNumRows ( db_result);
	msqlFreeResult(db_result);

    return (NumRows);

}

