/****************************************************************************
*
* File:             MSMonitor.c
* Author:           Nikolaos Radouniklis
* Date:             06/01/96
* Last changed:     07/24/96
* Description:      MSMonitor selects and excutes the appropriate MSMonitor
*					
*
****************************************************************************/

/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <msql.h>
#include <tcl.h>
#include <tk.h>


/* my includes */
#include "MS_Types.h"
#include "Constants.h"
#include "Database.h"
#include "TclInterp.h"
#include "util.h"

/* my defines */


/* type declarations */

/* function declarations */
int MSMonitorSelect (char *MSName , int MaxSamples, int Interval);
int MSMonitorVic ( int socket, int MaxSamples, int Interval, sigset_t *block_sigterm_p );
int MSMonitorVat ( int socket, int MaxSamples, int Interval, sigset_t *block_sigterm_p );
void MSMonitorVicKill ( int sig );
void MSMonitorVatKill ( int sig );

/* internal function declarations */
int MSUpdateParticipantList (char *MSName, char MSPartList[MAX_PARTICIPANTS][MAX_PARTICIPANT_LENGTH]);
int CheckCleanupOrSleep ( int socket, Tcl_Interp **interp, sigset_t *blockterm_signal_p, int Interval);
int VatMap ( char *CompressionS, int *CompressionN );
void MySleep (unsigned int usecs);
 


/************************************
Global Data Structures
************************************/



/****************************************************************************
*
* Function:         main
* Param:            [in] MSName (Name of Media Service)
*                   [in] MaxSamplesS (maximum number of samples kept in DB)
*                   [in] IntervalS (time interval in ms for monitoring)
* Returns:          
* Date:             06/06/96
* Last changed:     06/06/96
* Description:      Does all the monitoring stuff
*                   
****************************************************************************/

int main ( int argc, char *argv[] )
{
	
	char *MSName, *MaxSamplesS, *IntervalS;
	int  MaxSamples, Interval;

	MSName 		= (char *) malloc ( strlen(argv[1]) +1);
	MaxSamplesS=  (char *) malloc ( strlen(argv[2]) +1);
	IntervalS 	= (char *) malloc ( strlen(argv[3]) +1);
	
	strcpy (MSName, argv[1]);
	strcpy (MaxSamplesS, argv[2]);
	strcpy (IntervalS, argv[3]);

	MaxSamples = atoi (MaxSamplesS);	
	Interval   = atoi (IntervalS);	
	
	/* Call the MSM-dispatcher MSMonitorSelect */
	MSMonitorSelect ( MSName, MaxSamples, Interval );


	return (-1);

}




/****************************************************************************
*
* Function:         MSMonitorSelect
* Param:            [in] MSName (Name of Media Service)
*                   [in] MaxSamples (maximum number of samples kept in DB)
*					[in] Interval (time interval in ms for monitoring)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      1) Connects to CME database
*					2) Set the sigprogmask appropriately
*					3) Defines the MS-specific SIGTERM signal handler
*					4) Invokes the MS-specific monitor
*
****************************************************************************/

int MSMonitorSelect (char *MSName , int MaxSamples, int Interval) 
{

	sigset_t block_sigterm; /* sinalmask structure for blocking/unblocking */
	int socket; 	/* CME database socket descriptor */

	/* Initialize signal mask 					*/
	/* Add the SIGTERM signal to signal mask	*/
	/* Block the SIGTERM signal in signal mask  */ 
	sigemptyset (&block_sigterm);
  	sigaddset (&block_sigterm, SIGTERM);
  	sigprocmask (SIG_SETMASK, &block_sigterm, NULL);


	/* Connect to database */
	socket = DBConnect ( DBGetName () );

	
	/* invoke the MS-specific appropriate monitor */
	/* VIC */
	if (strcmp (MSName, VIC ) == 0)
	{
		signal (SIGTERM, MSMonitorVicKill );
		MSMonitorVic ( socket, MaxSamples, Interval, &block_sigterm );
	}	
	
	/* VAT */
	if (strcmp (MSName, VAT ) == 0)
	{
		signal (SIGTERM, MSMonitorVatKill);
		MSMonitorVat ( socket, MaxSamples, Interval, &block_sigterm );
	}	

	/* WB */
	if (strcmp (MSName, WB ) == 0)
	{
	}	

	return (-1);
}




/****************************************************************************
*
* Function:         MSMonitorVic
* Param:            [in] socket (Socket descriptor for database)
*                   [in] MaxSamples (maximum number of samples kept in DB)
*					[in] Interval (time interval in seconds for monitoring)
*					[in] block_sigterm_p (Blocking/Unblocking signals, e.g. SIGTERM)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     07/03/96
* Description:      collects Vic-Information and writes it into 
*					the VIC table in the CME database
* Tables:			
*					T_VicInfo
*
****************************************************************************/

int MSMonitorVic ( int socket, int MaxSamples, int Interval, sigset_t *block_sigterm_p ) 
{

	unsigned int counter=1; 	/* counts number of samples so far */
	char SQL_statement[SQL_STATEMENT_MAX];
	Tcl_Interp *interpr;

	struct VicInfo ParticipantSample;  /* structure holds Participant sample */
	char VicParticipantList[MAX_PARTICIPANTS][MAX_PARTICIPANT_LENGTH]; /*Vic PartList */

	int i;
	char TclCommand[TCL_STATEMENT_MAX];
	char *tempbuf; /*temporary buffer for monitored loss */
	int retval;
	
	/* allocate space for temporary buffer */
	tempbuf = (char *) malloc ( MAX_CHARBUFFER);

	/* Get TCL Interpreter for Tcl-procedures (defined in TclInterp.h) */
	GetTclInterp ( &interpr);	



	/* MONITOR ONLY IF APPLICATION IS ALIVE  */
    sprintf ( TclCommand, "Vic.IsAlive" );
    retval = 0;
    while ( retval == 0 )
    {
        Tcl_Eval (interpr, TclCommand);
        retval = atoi ( (*interpr).result);
    }
	
	/* enter now monitor loop and process until kill signal received */
	while (1)
	{

		/* go now through ParticipantList and write into T_VicInfo */
		/* 1. Update ParticipantList							   */
		/* 2. Delete old samples from T_VicInfo					   */
		/* 3. Insert new samples in T_VicInfo					   */

		
		/* 1. Update ParticipantList							   */
		MSUpdateParticipantList(VIC, VicParticipantList);

        /* 2. check if old sample has to be deleted */
        if ( counter > MaxSamples )   
        {
        	sprintf ( SQL_statement, "  DELETE FROM T_VicInfo\
                                        WHERE A_Counter = %d ",\
                                        counter - MaxSamples );
            msqlQuery (socket, SQL_statement);
        }



		/* 3. add information in T_VicInfo for each participant */
		i=0;
		while (VicParticipantList[i][0] != '\0')
		{
			sprintf (TclCommand, "Vic.GetInfo %s EncodingScheme", VicParticipantList[i]);
			Tcl_Eval (interpr, TclCommand);
			strcpy (ParticipantSample.Compression, (*interpr).result ); 
			sprintf (TclCommand, "Vic.GetInfo %s FPS", VicParticipantList[i]);
			Tcl_Eval (interpr, TclCommand);
			ParticipantSample.FrameRate = atof ((*interpr).result);
			sprintf (TclCommand, "Vic.GetInfo %s KBPS", VicParticipantList[i]);
			Tcl_Eval (interpr, TclCommand);
			ParticipantSample.Bandwith = atoi ((*interpr).result);
			sprintf (TclCommand, "Vic.GetInfo %s Loss", VicParticipantList[i]);
			Tcl_Eval (interpr, TclCommand);
			/* Loss format is (x.y%) --> extract first character ( */
			strcpy ( tempbuf, (*interpr).result);
			ParticipantSample.Loss = atoi (tempbuf+1);

       	   
       		/* insert new sample */
       	 	sprintf ( SQL_statement,"   INSERT INTO T_VicInfo\
       	   	                          ( A_Counter, A_Participant, \
										A_Compression, A_FrameRate, A_Bandwith, A_Loss)\
       	   	                          VALUES ( %d, '%s', '%s', %f, %d, %d )",\
       	   	                          counter,\
       	   	                          VicParticipantList[i],\
       	   	                          ParticipantSample.Compression,\
       	   	                          ParticipantSample.FrameRate,\
       	   	                          ParticipantSample.Bandwith,\
       	   	                          ParticipantSample.Loss ); 
       	 	msqlQuery (socket, SQL_statement);
	



			/* next participant */
			i+=1;	
		}


		/* Cleanup Or Sleep */
		CheckCleanupOrSleep ( socket, &interpr, block_sigterm_p, Interval);

		counter+=1;
	}

}


/****************************************************************************
*
* Function:         MSMonitorVat
* Param:			[in] socket (socket descriptor for database)
*                   [in] MaxSamples (maximum number of samples kept in DB)
*                   [in] Interval (time interval in seconds for monitoring)
*					[in] block_sigterm_p (Pointer for blocking SIGTERM signal)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     07/26/96
* Description:      collects Vat-Information and writes it into
*                   the VAT table in the CME database
*
****************************************************************************/
 
int MSMonitorVat ( int socket, int MaxSamples, int Interval, sigset_t *block_sigterm_p )
{


	unsigned int counter=1;     /* counts number of samples so far */
    char SQL_statement[SQL_STATEMENT_MAX];
    Tcl_Interp *interpr;
	char CompressionS[MAX_CHARBUFFER];	/* String variable holds Compression */
 
	char VatParticipantList[MAX_PARTICIPANTS][MAX_PARTICIPANT_LENGTH]; /*Vat PartList */
	struct VatInfo ParticipantSample;  /* structure holds Vat Participant sample */
	char TclCommand[TCL_STATEMENT_MAX];
	int i; /* counter for VatParticipantList */
	int retval; 

    /* Get TCL Interpreter for Tcl-procedures (defined in TclInterp.h) */
    GetTclInterp ( &interpr);    
 
    /* MONITOR ONLY IF APPLICATION IS ALIVE  */
    sprintf ( TclCommand, "Vat.IsAlive" );
    retval = 0;
    while ( retval == 0 )
    {
        Tcl_Eval (interpr, TclCommand);
        retval = atoi ( (*interpr).result);
    }
 
    /* enter now monitor loop and process until kill signal received */
    while (1)
    {
       
        /* go now through ParticipantList and write into T_VatInfo */
        /* 1. Update ParticipantList                               */
        /* 2. Delete old samples from T_VatInfo                    */
        /* 3. Insert new samples in T_VatInfo                      */

         
        /* 1. Update ParticipantList                               */
        MSUpdateParticipantList(VAT, VatParticipantList);

        /* 2. check if old sample has to be deleted */
        if ( counter > MaxSamples )
        {
            sprintf ( SQL_statement, "  DELETE FROM T_VatInfo\
                                        WHERE A_Counter = %d ",\
                                        counter - MaxSamples );
            msqlQuery (socket, SQL_statement);
        }
 
 
        /* 3. add information in T_VatInfo for each participant */
        i=0;
        while (VatParticipantList[i][0] != '\0')
        {
            sprintf (TclCommand, "Vat.GetInfo %s Encoding", VatParticipantList[i]);
            Tcl_Eval (interpr, TclCommand);

			strcpy ( CompressionS, (*interpr).result );
			VatMap ( CompressionS, &(ParticipantSample.Compression) );
          
            /* insert new sample */
            sprintf ( SQL_statement,"   INSERT INTO T_VatInfo\
                                      ( A_Counter, A_Participant, A_Compression )\
                                      VALUES ( %d, '%s', %d )",\
                                      counter,\
                                      VatParticipantList[i],\
                                      ParticipantSample.Compression );
            msqlQuery (socket, SQL_statement);
   
 
            /* next participant */
            i+=1;  

		}


        /* Cleanup Or Sleep */ 
        CheckCleanupOrSleep ( socket, &interpr, block_sigterm_p, Interval); 

        counter+=1;
    }

}
 
/****************************************************************************
*
* Function:         VatMap
* Param:            [in]  CompressionS (String e.g. GSM, DVI )
*                   [out] CompressionN (Numeric Code of Compression)     
* Returns:          (-1) OK, 0 Error
* Date:             07/08/96
* Last changed:     07/08/96
* Description:      Maps String Compression in Numeric Compression
*                   
*                   
* Example:         	VatMap ( GSM, Num) --> Num QOS_VAT_GOOD
*                   
****************************************************************************/

int VatMap ( char *CompressionS, int *CompressionN )
{
	
	static char tempbuf[4];

	/* Compare only first three characters ( only these are significant ) */
	strncpy (tempbuf, CompressionS, 3);
	tempbuf[3] = '\0';

	if ( strcmp (tempbuf, VAT_PCM ) == 0 )
		*CompressionN = QOS_VAT_PCM;
	if ( strcmp (tempbuf, VAT_DVI ) == 0 )
		*CompressionN = QOS_VAT_DVI;
	if ( strcmp (tempbuf, VAT_GSM ) == 0 )
		*CompressionN = QOS_VAT_GSM;
	if ( strcmp (tempbuf, VAT_LPC ) == 0 )
		*CompressionN = QOS_VAT_LPC;
	if ( strcmp (tempbuf, VAT_ZERO ) == 0 )
		*CompressionN = QOS_VAT_ZERO;

	return (-1);
}




/****************************************************************************
*
* Function:         MSMonitorVicKill
* Param:            [in] sig (signal which has to be processed
*                   	 in our case (SIGTERM)
* Returns:          
* Date:             06/05/96
* Last changed:     06/05/96
* Description:      1) clears T_VicInfo table in CME_DB
* 					2) kills the Vic monitoring process (itself)
*                   -) can't close local socket descriptor
* Problems:			Encountered problems: E.g. undefined behaviour
*					with CME database
****************************************************************************/

void MSMonitorVicKill ( int sig )
{
    int socket;     /* CME database socket descriptor */
    /* Connect to database */
    socket = DBConnect ( DBGetName () ); 
 
    DBClearTable (socket, "T_VicInfo");
    DBRelease (socket);
    
	printf ("MSG: MSMonitorVicKill %d\n", (int) getpid());
	raise ( sig );
}






/****************************************************************************
*
* Function:         MSMonitorVatKill
* Param:            [in] sig (signal which has to be processed
*                        in our case (SIGTERM)
* Returns:          
* Date:             06/05/96
* Last changed:     06/05/96
* Description:      1) clears T_VatInfo table in CME_DB
*					2) kills the Vat monitoring process (itself)
*                   
****************************************************************************/
 
void MSMonitorVatKill ( int sig )
{

	int socket;     /* CME database socket descriptor */
    /* Connect to database */
    socket = DBConnect ( DBGetName()); 

	DBClearTable (socket, "T_VatInfo");
	DBRelease (socket);
    
    printf ("MSG: MSMonitorVatKill %d\n", (int) getpid());
	raise ( sig );
}




/****************************************************************************
*
* Function:         MSUpdateParticipantList
* Param:            MSName: (Name of the Media Service (vic, vat))
*					MSPartList: (Array of paricipants)
*					
*                       
* Returns:			(-1) Update is OK; (0) if error
* Date:             06/25/96
* Last changed:     06/25/96
* Description:      Updates the Vic Participant List
*                   
****************************************************************************/

int MSUpdateParticipantList ( char *MSName, char MSPartList[MAX_PARTICIPANTS][MAX_PARTICIPANT_LENGTH] )
{
	int i;
    Tcl_Interp *interpr;
	static char TclParticipantList[MAX_PARTICIPANT_LIST];
	char *token;


	/* delete Participant List			  	  */
	for (i=0; i<MAX_PARTICIPANTS; i++)
		MSPartList[i][0] = '\0';
	
	/* create new Participant List 			  */
    /* Get TCL Interpreter for Tcl-procedures (defined in TclInterp.h) */
    GetTclInterp ( &interpr);

	/* VIC specific */
	if (strcmp ( MSName, VIC) == 0)
	{
    	/* If not already done, initialize Vic send variables */
    	Tcl_Eval (interpr, "Vic.SetCtrlMenu");

		/* invoke Tcl-Procedure to get Participant List */
    	Tcl_Eval (interpr, "Vic.GetParticipantList");
    	strcpy ( TclParticipantList, (*interpr).result );
	}

	/* VAT specific */
	if (strcmp (MSName, VAT) == 0)
	{
		/* invoke Tcl-Procedure to get Participant List */
		Tcl_Eval (interpr, "Vat.GetParticipantList");
		strcpy (TclParticipantList, (*interpr).result);
	}


	/* Proccess the TclParticipantList string and fill the MSPartList array */
	i=0;
	token = strtok ( TclParticipantList, DELIMITER );
	while (token !=NULL)
	{
		strcpy ( MSPartList[i], token );
		token = strtok (NULL, DELIMITER);
		i+=1;
	}	



	return(-1);
}




/****************************************************************************
*
* Function:         CheckCleanupOrSleep
* Param:            [in] socket: (database socket to be released)
*					[in] interpr: (Tcl-interpreter to be deleted)
*					[in] block_sigterm_p: signal mask
*					[in] Interval: Time interval to sleep
*
* Returns:          (-1) Cleanup; (0) Sleep
* Date:             07/03/96
* Last changed:     07/03/96
* Description:      Checks if any pending signals (SIGTERM) exist.
*					If so Cleanup (clear socket connection, delete
*					interpreter, unblock signal)
*					Otherwise sleep ( but not busy waiting )
*
* Side Effects:		When signal is unblocked, signal handler is called
*
****************************************************************************/

int CheckCleanupOrSleep ( int socket, Tcl_Interp **interp, sigset_t *block_sigterm_p, int Interval)
{

	sigset_t waiting_mask; /* data structure for waiting signals */

	/* Check if pending signals exist */
	sigpending (&waiting_mask);

    if (sigismember (&waiting_mask, SIGTERM))
	{

		/* SIGTERM is sent: --> Cleanup */
		DBRelease (socket);
		Tcl_DeleteInterp (*interp);
		
		/* Unblock the SIGTERM signal --> signal handling function is invoked */
		sigprocmask (SIG_UNBLOCK, block_sigterm_p, NULL);

		/* return Cleanup */	
		return ( -1);
	}

	/* no pending signal --> sleep */
	MySleep (Interval);
	return (0);
		
}


/* Nonblocking sleep */
void MySleep (unsigned int usecs)
{
    struct timeval tv;
    tv.tv_sec  = usecs / 1000000L;
    tv.tv_usec = usecs % 1000000L;
    select (0, 0, 0, 0, &tv);
}
 

