/****************************************************************************
*
* File:             QoSMapper.c
* Author:           Nikolaos Radouniklis
* Date:             06/25/96
* Last changed:     07/08/96
* Description:      QoS Mapper maps qualities in different levels
*					1) User level
*					2) Application level
*					3) Resource level
*
****************************************************************************/



/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <tk.h>
#include <tcl.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"

/* my defines */
/* Array order of QoS attributes */ 
#define ASC                     0
#define DESC                    1


/* prototypes external */
int VicApp2User ( VicAppStruct* VicApp, int MappingStrategy, float *QoSValue);
int VicUser2App ( float QoSLevel, VicAppStruct* VicApp );
int VatApp2User ( VatAppStruct *VatApp, int MappingStrategy, float* QoSVat);
int VatUser2App ( float QoSLevel, VatAppStruct* VatApp );


/* prototypes internal */
int GetIndex ( float Value, int Array[][QOS_LEVELS], int Row, float* QoSValue );
int GetAppParam ( float QoSLevel, int Array[][QOS_LEVELS], int Row , float *AppParam);





/* static data structures QOS-Mapping tables */

/*********************************************
* MappingTable: MT_VicJPEGApp2User 
* 1.Row: 		FrameRate
* 2.Row:		Resolution
*
**********************************************/
static int MT_VicJPEGApp2User [][QOS_LEVELS] = {
	{ 1,  3,  6, 15, 25 }, 
	{ 3, 19, 34, 49, 63 }
};

/*********************************************
* Coefficient Matrix: 	CM_VicJPEGApp2User 
* 1.Value: 				FrameRate
* 2.Value:				Resolution
**********************************************/
static int CM_VicJPEGApp2User [] = { 1, 1 };




/*********************************************
* MappingTable: MT_VicH261App2User  
* 1.Row:        FrameRate
* 2.Row:        Resolution
*
**********************************************/
static int MT_VicH261App2User [][QOS_LEVELS] = {
    {  1,  3,  6, 15, 25 }, 
    { 3, 19, 34, 49,  63 }   
}; 
 
/*********************************************
* Coefficient Matrix:   CM_VicH261App2User  
* 1.Value:              FrameRate
* 2.Value:              Resolution
**********************************************/
static int CM_VicH261App2User [] = { 1, 1 };






/*********************************************
* MappingTable: MT_VatApp2User
* 1.Row:        Encoding
*
**********************************************/
static int MT_VatApp2User [][QOS_LEVELS] = {
	{ QOS_VAT_BAD, QOS_VAT_LPC, QOS_VAT_GSM, QOS_VAT_DVI, QOS_VAT_PCM},  
};

/*********************************************
* Coefficient Matrix:   CM_VatApp2User
* 1.Value:              FrameRate
* 2.Value:              Resolution
**********************************************/
static int CM_VatApp2User [] = { 1 };






/****************************************************************************
*
* Function:         VicApp2User
* Param:			[in] VicApp (Vic Structure) 
*					-- [in] Encoding (e.g. JPEG, H261)
*					-- [in] FrameRate  (QOS_NO_VALUE means not taken into consideration)
*					-- [in] Resolution (QOS_NO_VALUE means not taken into consideration)
*					[in] MappingStrategy (strategy chosen for mapping)
*					[out] QoSVic (exact QoS value OR! QOS_ZERO)
*
* Returns:			Discrete Value between QOS_BAD to QOS_EXCELLENT
* Date:             06/25/96
* Last changed:     07/08/96 (Coefficient Matrix implemented)
* Description:      translates application parameters to user parameters
* Example:			(FrameRate: 15, Resolution: 70) --> Quality: 4, --> QoSValue: 4.45			
*					
* Comment:			Coefficient matrix for weighing different
*					application attributes is not implemented yet
*
****************************************************************************/

int VicApp2User ( VicAppStruct *VicApp, int MappingStrategy, float* QoSVic)
{
	
	int QoSDiscrete		= QOS_NO_VALUE;
	float QoSFrameRate  = (float) QOS_NO_VALUE;
	float QoSResolution = (float) QOS_NO_VALUE;


	/* set QoSVic to QOS_ZERO */
	*QoSVic = (float) QOS_ZERO;

	/*  JPEG Encoding Scheme */
	/*	Compute independently the User QoS of the different parameters  */ 
	if (strcmp ( (*VicApp).Encoding, JPEG) == 0)
	{
		if ((*VicApp).FrameRate != QOS_NO_VALUE)	
			GetIndex (  (*VicApp).FrameRate, MT_VicJPEGApp2User, QOS_ROW_FRAMERATE, &QoSFrameRate );
		if ((*VicApp).Resolution != QOS_NO_VALUE)
			GetIndex (  (*VicApp).Resolution, MT_VicJPEGApp2User, QOS_ROW_RESOLUTION, &QoSResolution);
	}


    /*  H261 Encoding Scheme */
    /*  Compute independently the User QoS of the different parameters  */
    if (strcmp ( (*VicApp).Encoding, H261) == 0)
    {
        if ((*VicApp).FrameRate != QOS_NO_VALUE)
            GetIndex (  (*VicApp).FrameRate, MT_VicH261App2User, QOS_ROW_FRAMERATE, &QoSFrameRate );
        if ((*VicApp).Resolution != QOS_NO_VALUE)
            GetIndex (  (*VicApp).Resolution, MT_VicH261App2User, QOS_ROW_RESOLUTION, &QoSResolution);
    }   



	/* Compute overall QoS (QoSVic) depending on MappingStrategy */

	/* a) take the minimum */
	if (MappingStrategy == QOS_MAP_MIN)
	{
		if ( QoSFrameRate != QOS_NO_VALUE)
			*QoSVic = QoSFrameRate;
		
		if ( QoSResolution != QOS_NO_VALUE)
			if (*QoSVic != QOS_NO_VALUE)
				*QoSVic = (( *QoSVic < QoSResolution) ? *QoSVic : QoSResolution);
			else
				*QoSVic = QoSResolution;
	}

	/* b) take the maximum */	
    if (MappingStrategy == QOS_MAP_MAX)
    {
        if ( QoSFrameRate != QOS_NO_VALUE) 
            *QoSVic = QoSFrameRate;  
         
        if ( QoSResolution != QOS_NO_VALUE)
            if (*QoSVic != QOS_NO_VALUE) 
                *QoSVic = (( *QoSVic > QoSResolution) ? *QoSVic : QoSResolution); 
            else 
                *QoSVic = QoSResolution; 
    }  



	/* c) take the medium */
	if (MappingStrategy == QOS_MAP_MED)
	{
		int n= 0; /* number of attributes */
		float QoSVicSum = 0.0;
	
		if ( QoSFrameRate != QOS_NO_VALUE )
		{
			if (strcmp ( (*VicApp).Encoding, JPEG) == 0)
			{
				QoSVicSum =  QoSVicSum + CM_VicJPEGApp2User[QOS_ROW_FRAMERATE]* QoSFrameRate;
				n+= CM_VicJPEGApp2User[QOS_ROW_FRAMERATE];
			}
            if (strcmp ( (*VicApp).Encoding, H261) == 0) 
            { 
                QoSVicSum =  QoSVicSum + CM_VicH261App2User[QOS_ROW_FRAMERATE]* QoSFrameRate;
                n+= CM_VicH261App2User[QOS_ROW_FRAMERATE];
            }

		}

		if (QoSResolution != QOS_NO_VALUE)
		{
			if (strcmp ( (*VicApp).Encoding, JPEG) == 0)
			{
				QoSVicSum =  QoSVicSum + CM_VicJPEGApp2User[QOS_ROW_RESOLUTION]* QoSResolution;
				n+=CM_VicJPEGApp2User[QOS_ROW_RESOLUTION];
			}
			if (strcmp ( (*VicApp).Encoding, H261) == 0)
			{
				QoSVicSum =  QoSVicSum + CM_VicH261App2User[QOS_ROW_RESOLUTION]* QoSResolution;
				n+=CM_VicH261App2User[QOS_ROW_RESOLUTION];
			}
		}
		
		if ( n != 0)
			*QoSVic = QoSVicSum/n;
	}


	/* compute discrete value now (QoSDiscrete) */
	QoSDiscrete = (int) (*QoSVic+0.5);

	/* return user level quality */
	return (QoSDiscrete);	
}







/****************************************************************************
*
* Function:         VicUser2App
* Param:            [in] QoSLevel (exact QoS value)
*                   [in out] VicApp (structure to be filled with values)  
*					-- [in]  Encoding (optional)
*					-- [out] FrameRate
*					-- [out] Resolution
*
* Returns:          (-1) OK, (0) Error
* Date:             06/28/96
* Last changed:     07/22/96 
* Description:      translates user parameters to application parameters
* Example:          Quality: 4.5 --> (FrameRate: 20, Resolution: 70)
*
*
****************************************************************************/
 
int VicUser2App ( float QoSLevel, VicAppStruct* VicApp )
{
	int *MT_Table; 	/* generic mapping table pointer */
	
    /*  Setup the appropriate Mapping Table MT_* depending on the Encoding scheme */
    if (strcmp ( (*VicApp).Encoding, JPEG) == 0)
		MT_Table = ( void *) MT_VicJPEGApp2User;
    else if (strcmp ( (*VicApp).Encoding, H261) == 0)
		MT_Table = ( void *) MT_VicH261App2User;
	else
	{
		/* if no encoding scheme specified use default JPEG table */
		MT_Table = (void *) MT_VicJPEGApp2User;
	}

	/* fill now the structure fields ( FrameRate, Resolution ) */
	GetAppParam ( QoSLevel, (void *) MT_Table, QOS_ROW_FRAMERATE, &((*VicApp).FrameRate) ); 
	GetAppParam ( QoSLevel, (void *) MT_Table, QOS_ROW_RESOLUTION, &((*VicApp).Resolution) ); 
 

	return (-1);

}







/****************************************************************************
*
* Function:         VatApp2User
* Param:            [in] VatApp (Vat Structure)
*                   -- [in] Encoding (e.g. DVI, GSM)
*                   [in] MappingStrategy (strategy chosen for mapping)
*                   [out] QoSVat (exact QoS value) OR QOS_ZERO
*
* Returns:          Discrete Value between QOS_BAD to QOS_EXCELLENT
* Date:             07/01/96
* Last changed:     07/01/96 
* Description:      translates application parameters to user parameters
* Example:          (Encoding Scheme: DVI) --> Quality: 4, --> QoSValue: 4
*
****************************************************************************/
 
int VatApp2User ( VatAppStruct *VatApp, int MappingStrategy, float* QoSVat)
{
   
    int QoSDiscrete     = QOS_NO_VALUE;
    float QoSEncoding  = (float) QOS_NO_VALUE;


	/* set QoSVat to QOS_ZERO */
	*QoSVat = (float) QOS_ZERO;

 
    if ((*VatApp).Encoding != QOS_NO_VALUE)
    	GetIndex ( (*VatApp).Encoding, MT_VatApp2User, QOS_ROW_VAT_ENCODING, &QoSEncoding );
 
 
    /* Compute overall QoS (QoSVat) depending on MappingStrategy */
 
    /* a) take the minimum */
    if (MappingStrategy == QOS_MAP_MIN)
    {
        if ( QoSEncoding != QOS_NO_VALUE)
            *QoSVat = QoSEncoding;
    }

 
    /* b) take the maximum */
    if (MappingStrategy == QOS_MAP_MAX)
    {
        if ( QoSEncoding != QOS_NO_VALUE)
            *QoSVat = QoSEncoding;
	}


    /* c) take the medium */
    if (MappingStrategy == QOS_MAP_MED)
    {
        int n= 0; /* number of attributes */
        float QoSVatSum = 0.0;

        if ( QoSEncoding != QOS_NO_VALUE )
        {
            QoSVatSum =  QoSVatSum + CM_VatApp2User[QOS_ROW_VAT_ENCODING]* QoSEncoding;
            n+= CM_VatApp2User[QOS_ROW_VAT_ENCODING];
        }

        if ( n != 0)
            *QoSVat = QoSVatSum/n;
    }


    /* compute discrete value now (QoSDiscrete) */
    QoSDiscrete = (int) (*QoSVat+0.5);

 
    /* return user level quality */
    return (QoSDiscrete);
}







/****************************************************************************
*
* Function:         VatUser2App
* Param:            [in] QoSLevel (exact QoS value)
*                   [out] VatApp (structure to be filled with values)
*                   -- [out]  Encoding
*
* Returns:          (-1) OK, (0) Error
* Date:             07/01/96
* Last changed:     07/01/96
* Description:      translates user parameters to application parameters
* Example:          Quality: 4 --> (Encoding: QOS_VAT_DVI)
*
*
****************************************************************************/

int VatUser2App ( float QoSLevel, VatAppStruct* VatApp )
{
   
    /* fill the structure fields ( Encoding ) */
    GetAppParam ( QoSLevel, MT_VatApp2User, QOS_ROW_VAT_ENCODING, &((*VatApp).Encoding) );

    return (-1); 

}
 
 
 


/****************************************************************************
*
* Function:         GetIndex
* Param:            [in]  Value (value to be searched in list)
*					[in]  Array (list of values to parse) 
*					[in]  Row (Row in array to be considered)
*					[out] QoSValue (retrieves the decimal QoSValue, e.g. 2.7)
* 
* Returns:          Index (Index in list )
* Date:             06/25/96
* Last changed:     06/25/96
* Description:      GetIndex parses the list of the transmitted static 
*					array and returns the index of the element
*					closest to Value
* Example:          
*
****************************************************************************/

int GetIndex ( float Value, int Array[][QOS_LEVELS], int Row , float *QoSValue)
{

	int Column; /* Column to carry the result */
	int Order; 	/* Order of list in array */
    int dxdy, y0;  /*Interpolation variables */

	/* check if list is asc or desc */
	if (Array[Row][0] < Array[Row][QOS_LEVELS-1] )
		Order = ASC;
	else
		Order = DESC;

			
	/* ASCending order */
	if ( Order == ASC)
	{
		/* set starting element in list */
		Column = 0;

		/* Scan through list until Column matches value or end is reached */
		while (((float )Array[Row][Column] <= Value) && (Column <QOS_LEVELS))
			Column+=1;	
	}

	/* DESCending order */
	if (Order == DESC )
	{
	    /* set starting element in list */
        Column = 0;
 
        /* Scan through list until Column matches value or end is reached */
        while (((float )Array[Row][Column] >= Value) && (Column <QOS_LEVELS))
            Column+=1;
    }


    /* Retrieve now the exact QoS Value */
    if (Column == QOS_LEVELS)
        *QoSValue = (float) QOS_EXCELLENT;   
    else if (Column >= QOS_BAD) 
    {    
        dxdy = Array[Row][Column] - Array[Row][Column-1]; 
        y0 = Array[Row][Column-1] - (dxdy * Column);
         
        *QoSValue = ( Value - (float) y0 ) / (float) dxdy; 
    }    
	else if (Column == 0)
	{
		dxdy = Array[Row][Column] - 0;
		y0 = 0;
		*QoSValue = ( Value - (float) y0 ) / (float) dxdy;
	}




    /* return Column == index == Quality level */
    return (Column);
}







/****************************************************************************
*
* Function:         GetAppParam
* Param:            [in]  OoSLevel (usually Index+1)
*					[in]  Array (list of values to parse)
*                   [in]  Row (Row in array to be considered)
*                   [out] AppParam (e.g. FrameRate = 23.6 )
*
* Returns:          (-1) ok, (0) error
* Date:             06/28/96
* Last changed:     07/01/96
* Description:      GetAppParam parses the list of the transmitted static
*                   array and fills AppParam with the interpolated value
*                   closest to Value
* Example:			GetAppParam ( 3.4, MT...., ROW_FRAMERATE, AppParam) -> 23.6
*
****************************************************************************/
 
int GetAppParam ( float QoSLevel, int Array[][QOS_LEVELS], int Row , float *AppParam)
{

	int p1x, p1y, p2x, p2y; /* 2 points in the interpolated line */
	int dxdy, y0;			/* interpolation variables */


	/* Compute the 2 relevant points in QoS Mapping table */	
	if ( (int) QoSLevel != QOS_ZERO )
	{
		if ( (int) QoSLevel == QOS_EXCELLENT )
			p2x = QOS_EXCELLENT; 
		else
			p2x = (int) QoSLevel + 1;

		p1x = p2x -1;
		p1y = Array[Row][p1x -1]; /* due to index transformation */
		p2y = Array[Row][p2x -1]; /* due to index transformation */
	}
	else
	{
		/* take as (p1x,p1y) (0,0) */
		p1x = QOS_ZERO;
		p1y = QOS_ZERO;
		p2x = QOS_ZERO +1;
		p2y = Array[Row][p2x -1]; /* due to index transformation */
	}
	
	/* Compute interpolation function */	
    dxdy = p2y - p1y;
    y0 = p1y - dxdy * p1x;


	/* Compute the exact value of the application parameter */
	*AppParam = (float) dxdy * QoSLevel + (float) y0;


	return (-1);

}





