/****************************************************************************
*
* File:             ServiceManager.c
* Author:           Nikolaos Radouniklis
* Date:             05/16/96
* Last changed:     06/24/96
* Description:      ServiceManager controls Media Services, Monitors, etc.
*
*
****************************************************************************/

/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <msql.h>
#include <tcl.h>
#include <tk.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"
#include "Database.h"
#include "TclInterp.h"

/* my defines */


/* type declarations */

/* function declarations */

int StartMS ( char *MSName, char *Filename, char *ArgumentList[] );
int StopMS  (char *MSName);
int StartMSMonitor (char *MSName, int MaxSamples, int Interval);
int StopMSMonitor  (char *MSName);
int SetMSParameter (char *MSName, char* Parameter, char* Value);
int GetMSParameterNum (char* Table, char* Attr, int Samples, char* Condition, float *Value);
int GetMSParameterString (char* Table, char* Attr, char* Condition, char* String);


/* internal function declarations */
int RegisterActiveMS  (int pid, char *MSName);
int RegisterActiveMSM (int pid, char *MSName);
int UnregisterActiveMS ( char *MSName );
int UnregisterActiveMSM ( char *MSName );
int GetTclInterp ( Tcl_Interp **interpr_addr );
int SetMSParameterVic (char *Attr, char* Value);
int SetMSParameterVat (char *Attr, char* Value);



/* global variables */

extern int sock;               /* socket descriptor for CME database access */
extern Tcl_Interp *interp;





/****************************************************************************
*
* Function:         StartMS
* Param:			[in] MSName ( Name of the Media Service )
*					[in] Filename ( Filename of the Media Service )
* 					[in] ArgumentList (e.g. "vic", "crawdad/3456", NULL )
* Returns:			-1 (OK) , 0 (Error)
* Date:             05/24/96
* Last changed:     05/24/96
* Description:      1) Starts the appropriate media service
*					1) Initializes the ms
*					2) Register media service
*					3) Starts the corresponding media monitor
*
****************************************************************************/

int StartMS ( char *MSName, char *Filename, char *ArgumentList[] )
{
    
	Tcl_Interp  *interpr; /* tcl interpreter for commands 	*/
	int status=-1; 	/* indicates if fork succeded or not 	*/
	pid_t pid;  	/* process PID 					   	 	*/
	int retval; 	/* check if is alive					*/


	/* fork a new process for the MS */
	pid = fork ();
	if (pid == 0)
	{
		/* This is the child process. Execute a shell and invoke the MS */
		execvp ( Filename, ArgumentList ); 
		_exit (EXIT_FAILURE);
	}	
  	else if (pid < 0)
	{
    	/* The fork failed. MS not invoked */
    	status = 0;
	}
  	else
	{
    	/* This is the parent process. 								*/
		/* Write the MS PID Number in the list of active MS		 	*/
		RegisterActiveMS ( pid, MSName );

		/* But wait a while before starting the MSMonitor 			*/
		sleep ( MAX_INTERVAL);	

		/* Perform now the initialization of the ms	(VIC,VAT,WB)	*/
		/* ... but get first a tcl interpreter						*/
    	/* Get TCL Interpreter for Tcl-procedures */
    	GetTclInterp ( &interpr );
		
		if (strcmp (VIC, MSName) == 0 )
		{
			/* INITIALIZE ONLY IF APPLICATION IS ALIVE  */
			retval = 0;
        	while ( retval == 0 )
        	{
           		Tcl_Eval (interpr, "Vic.IsAlive");
           	 	retval = atoi ( (*interpr).result);
        	}
			/* 1. transmit enabled		*/
			/* 2. jpeg encoding scheme  */
			/* 3. open bandwith to max 	*/
			sleep ( MAX_INTERVAL);	
			Tcl_Eval ( interpr, "Vic.SetTransmit" );
			Tcl_Eval ( interpr, "Vic.SetEncoding jpeg" );
			Tcl_Eval ( interpr, "Vic.SetMyMaxKBPS 3072" );
			
		}
		else if (strcmp (VAT, MSName) == 0 )
		{
 			/* INITIALIZE ONLY IF APPLICATION IS ALIVE  */
            retval = 0; 
            while ( retval == 0 )
            {
                Tcl_Eval (interpr, "Vat.IsAlive"); 
                retval = atoi ( (*interpr).result);  
            } 
			/* 1. set default (talk)		*/
			sleep ( MAX_INTERVAL);	
			Tcl_Eval (interpr, "Vat.SetButton Talk" );		
		}
		else if (strcmp (WB, MSName) == 0 )
		{
		}

		/* Start now the corresponding Media Service Monitor (MSM)  */
		/* only for services which support MSM 						*/
		if ( (strcmp (MSName, VIC) == 0) || \
			 (strcmp (MSName, VAT) == 0) )
		{
	 		StartMSMonitor (MSName, MAXSAMPLES, INTERVAL);
		}
	}



	return status;
}






/****************************************************************************
*
* Function:         StopMS
* Param:            [in] MSName 
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     07/03/96
* Description:      3) Stops the active media service
*                   2) Unregisters the media service
*                   1) Stops the corresponding media monitor
*
****************************************************************************/
 

int StopMS  (char *MSName)
{

	int			retval;
	Tcl_Interp  *interpr;


    /* Stop now the corresponding Media Service Monitor (MSM)  */
    /* only for services which support MSM                      */
    if (  (strcmp (MSName, VIC) == 0) ||\
          (strcmp (MSName, VAT) == 0) )
    {
		StopMSMonitor ( MSName );
		/* Sleep until MSM really died */
		sleep (MAX_INTERVAL);
    }


	/* Unregister the media service */
	retval = UnregisterActiveMS ( MSName );
	if (retval == 0)
		return (0);

  	/* Get TCL Interpreter for Tcl-procedures */
	GetTclInterp ( &interpr );


	/* Stop the active MS (e.g. by invoking the right TCL-Procedure) */
	/* ... OR killing the process									 */
	/* VIC */
	if (strcmp (MSName, VIC ) == 0)
	{
		retval = Tcl_Eval (interpr, "Vic.Quit");
	}	
	
	/* VAT */
	if (strcmp (MSName, VAT ) == 0)
	{
		retval = Tcl_Eval (interpr, "Vat.Quit");
	}	

	/* WB */
	if (strcmp (MSName, WB ) == 0)
	{
		/* kill ( pid, sigtype ) */
		kill ( retval, SIGTERM ); 
	}	


	return(-1);
}






/****************************************************************************
*
* Function:         GetMSParameterNum
* Param:            [in] TableName 
*					[in] Attr ( Name of attribute)
*                   [in] Samples (number of last samples to be considered)
*					[in] Condition (e.g. select a certain participant)
*					[out] Value (value of param)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/06/96
* Last changed:     06/26/96
* Description:      retrieves the parameter value of <Attr>
*					regarding the last <Samples>
*
****************************************************************************/

int GetMSParameterNum (char* Table, char* Attr, int Samples, char* Condition, float *Value)
{
	
	/* Retrieve the average value by calling DBAverage */
	/* remember: sock is a global variable			   */		
	DBAverage (sock, Table, Attr, Samples, Condition, Value );

	return(-1);
}



/****************************************************************************
*
* Function:         GetMSParameterString
* Param:            [in] TableName
*                   [in] Attr ( Name of attribute)
*                   [in] Condition (e.g. select a certain participant)
*                   [out] Value (Value of String)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/26/96
* Last changed:     06/26/96
* Description:      retrieves the parameter string value of <Attr>
*
****************************************************************************/

int GetMSParameterString (char* Table, char* Attr, char* Condition, char* Value)
{
   
    /* Retrieve the string value by calling DBGetAttr */
    /* remember: sock is a global variable            */
    DBGetAttr (sock, Table, Attr, Condition, Value );

    return(-1);  
}






/****************************************************************************
*
* Function:         SetMSParameter
* Param:            [in] MSName (Name of media service) 
*                   [in] Attr (Name of attribute)
*                   [in] Value (value of param)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/24/96
* Last changed:     06/24/96
* Description:      sets the value of the attribute <Attr>
*					by dispatching to the appropriate set function
*
****************************************************************************/

int SetMSParameter (char *MSName, char* Attr, char* Value)
{



	/* invoke the MS-specific appropriate set function */
    /* VIC */
    if (strcmp (MSName, VIC ) == 0)
    {
       SetMSParameterVic ( Attr, Value ); 
    }

    /* VAT */
    if (strcmp (MSName, VAT ) == 0)
    {
       	SetMSParameterVat ( Attr, Value ); 
    }
 
    /* WB */
    if (strcmp (MSName, WB ) == 0)
    {
    }


	return (-1);
}



/****************************************************************************
*
* Function:         SetMSParameterVic
* Param:            [in] Attr (Name of attribute)
*                   [in] Value (value of param)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/24/96
* Last changed:     06/24/96
* Description:      sets the value of the attribute <Attr>
*
****************************************************************************/

int SetMSParameterVic (char *Attr, char* Value)
{

	char TclStatement[TCL_STATEMENT_MAX];
	
    /* Get TCL Interpreter for Tcl-procedures (defined in TclInterp.h) */
	/* !Remember: interp is a global variable						   */
    GetTclInterp ( &interp);
 
    /* If not already done, initialize Vic send variables */
    Tcl_Eval (interp, "Vic.SetCtrlMenu");

	/* set now the attribute by evaluating the Tcl- Script			   */
	/* and invoking the appropriate Tcl-procedure					   */

	/* set max. framerate */
	if ( strcmp (Attr, VIC_MAXFPS ) == 0 )
	{
		sprintf ( TclStatement, "Vic.SetMyMaxFPS %s", Value );
	}

    /* set max. bandwith */
    if ( strcmp (Attr, VIC_MAXKBPS ) == 0 )   
    {
        sprintf ( TclStatement, "Vic.SetMyMaxKBPS %s", Value ); 
    }

    /* set encoding scheme */
    if ( strcmp (Attr, VIC_ENCODING ) == 0 )   
    {
        sprintf ( TclStatement, "Vic.SetEncoding %s", Value ); 
    }

    /* set resolution */
    if ( strcmp (Attr, VIC_RESOLUTION ) == 0 )
    {
        sprintf ( TclStatement, "Vic.SetQuality %s", Value );
    }


	/* execute Tcl command */	
    Tcl_Eval (interp, TclStatement );


	return (-1);
}



/****************************************************************************
*
* Function:         SetMSParameterVat
* Param:            [in] Attr (Name of attribute)
*                   [in] Value (value of param)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/24/96
* Last changed:     06/24/96
* Description:      sets the value of the attribute <Attr>
*
****************************************************************************/
 
int SetMSParameterVat (char *Attr, char* Value)
{
    
    char TclStatement[TCL_STATEMENT_MAX]; 
    
    /* Get TCL Interpreter for Tcl-procedures (defined in TclInterp.h) */
    /* !Remember: interp is a global variable                          */
    GetTclInterp ( &interp);
 
    /* If not already done, initialize Vic send variables */
    Tcl_Eval (interp, "Vic.SetCtrlMenu");
 
    /* set now the attribute by evaluating the Tcl- Script             */
    /* and invoking the appropriate Tcl-procedure                      */
 
    /* set encoding scheme */
    if ( strcmp (Attr, VAT_ENCODING ) == 0 )    
    { 
        sprintf ( TclStatement, "Vat.SetEncoding %s", Value );  
    } 
 
    /* execute Tcl command */    
    Tcl_Eval (interp, TclStatement );
 
 
    return (-1); 
}







/****************************************************************************
*
* Function:         StartMSMonitor
* Param:            [in] MSName (Name of the Media Service)
*                   [in] MaxSamples (Maximum of samples kept in buffer)
*					[in] Interval (monitoring interval in ms)
* Returns:          -1 (OK) , 0 (Error)
* Date:             05/29/96
* Last changed:     05/29/96
* Description:      1) Starts the MSM 
*
****************************************************************************/

int StartMSMonitor (char *MSName, int MaxSamples, int Interval)
{

	int status=-1; 	/* indicates if fork succeded or not */
	pid_t pid;  	/* process PID 					   	 */

	char MaxSamplesS[10]; /* String as command line param */
	char IntervalS  [10]; /* String as command line param */

	/* fork a new process for the MSM */
	pid = fork ();
	if (pid ==0)
	{
		/* Child process:  Monitor exexutable: MSMONITOR 	*/
		/* Param: <MSName> <MaxSamples> <Interval>			*/
		sprintf ( MaxSamplesS, "%d", MaxSamples);
		sprintf ( IntervalS, "%d", Interval);
		execl ( MSMONITOR_FILE, MSMONITOR_EXEC, MSName, MaxSamplesS, IntervalS, NULL);
	}
	else if (pid < 0)
	{
		/* fork failed. MSM not invoked */
		status = 0;
	}
	else
	{
    	/* This is the parent process. 								*/
		/* Write the MSM PID Number in the list of active MSM	 	*/
		RegisterActiveMSM ( pid, MSName );

	}	


	return status;	
}
	





/****************************************************************************
*
* Function:         StopMSMonitor
* Param:            [in] MSName
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      1) Unregisters the MSM
*					2) Stops the appropriate MS Monitor
*                   
****************************************************************************/
 
 
int StopMSMonitor (char *MSName)
{
	int pid; /* process ID of MS Monitor */

	/* Unregister the media service monitor */
	pid = UnregisterActiveMSM ( MSName );

	/* Stop the monitoring process */
	/* Send kill signal to child process */
	if (pid != 0)
		kill ( (pid_t) pid, SIGTERM );

	return(-1);

}








/****************************************************************************
*
* Function:         RegisterActiveMS
* Param:            [in] Pid (Process ID)
*                   [in] MSName (Media Service Name)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Registers the invoked MS in the CME database
*
****************************************************************************/

int RegisterActiveMS ( int pid, char *MSName )
{

	char SQL_statement[SQL_STATEMENT_MAX]; 	

	/* generate dynamic SQL-statement */	
	sprintf ( SQL_statement,"	INSERT INTO T_ActiveMS \
								VALUES ( %d, '%s')",   \
								pid, MSName );

	/* insert pid and media service name into database */	
	if (msqlQuery (sock, SQL_statement) == -1 )
        error_handler();


	return (-1);
}
	



/****************************************************************************
*
* Function:         UnregisterActiveMS
* Param:            [in] MSName (Media Service Name)
* Returns:          Pid of registered process; (0) if error
* Date:             06/04/96
* Last changed:     07/18/96
* Description:      Unregisters the invoked MS from the CME database
*
****************************************************************************/

int UnregisterActiveMS ( char *MSName )
{


	char SQL_statement[SQL_STATEMENT_MAX]; 	
    m_result    *db_result;
    m_row       db_row;
    int         pid;

	/* Unregister the media service */
    /* a) Retrieve PID */
    sprintf ( SQL_statement,"   SELECT A_Pid FROM T_ActiveMS    \
                                WHERE A_MSName  = '%s'",        \
                                MSName );
    msqlQuery (sock, SQL_statement);
    db_result = msqlStoreResult(); 
    db_row    = msqlFetchRow (db_result);

	if (db_row == NULL )
		return (0);

    pid = atoi (db_row[0]);

    /* b) remove row */
    sprintf (SQL_statement, "   DELETE FROM T_ActiveMS  \
                                WHERE A_Pid = %d",      \
                                pid );
    msqlQuery (sock, SQL_statement);

	return(pid);

}




/****************************************************************************
*
* Function:         RegisterActiveMSM
* Param:            [in] Pid (Process ID)
*                   [in] MSName (Media Service Name)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Registers the invoked MSM in the CME database
*
****************************************************************************/
 
int RegisterActiveMSM ( int pid, char *MSName )
{
    
    char SQL_statement[SQL_STATEMENT_MAX];   
 
    /* generate dynamic SQL-statement */
    sprintf ( SQL_statement,"   INSERT INTO T_ActiveMSM	\
                                VALUES ( %d, '%s')",	\
                                pid, MSName );  
 
    /* insert pid and media service name into database */   
    if (msqlQuery (sock, SQL_statement) == -1 )
        error_handler();   
 
 
    return (-1);   
} 




/****************************************************************************
*
* Function:         UnregisterActiveMSM
* Param:            [in] MSName (Media Service Name)
* Returns:          pid ( process ID of monitoring process )
* Date:             06/04/96
* Last changed:     06/04/96
* Description:      Unregisters the invoked MS Monitor from the CME database
*
****************************************************************************/
 
int UnregisterActiveMSM ( char *MSName )
{

	char SQL_statement[SQL_STATEMENT_MAX];
    m_result    *db_result;
    m_row       db_row;
    int         pid;

    /* Unregister the media service monitor */
    /* Retrieve PID */
    sprintf ( SQL_statement,"   SELECT A_Pid FROM T_ActiveMSM   \
                                WHERE A_MSName  = '%s'",        \
                                MSName );
    msqlQuery (sock, SQL_statement);
    db_result = msqlStoreResult();
    db_row    = msqlFetchRow (db_result);
	if (db_row == NULL)
		return (0);

    pid = atoi (db_row[0]);

    /* Remove row now */
    sprintf (SQL_statement, "   DELETE FROM T_ActiveMSM \
                                WHERE A_Pid = %d",      \
                                pid );
    msqlQuery (sock, SQL_statement);

	return (pid);

}

