# *************************************************************************** 
# 
# File:             vic.interface.tcl
# Author:           Nikolaos Radouniklis
# Date:             05/04/96
# Last changed:     06/26/96
# Description:      bunch of Tcl/Tk procedures
#                   to manipulate the vic
#                   video application v2.7a
# 
# *************************************************************************** 
 




# *************************************************************************** 
#   
# Procedure:        Vic.GetApplicationName
# Date:             05/06/96 
# Param:            - 
# Result:           $ApplicationName
# Description:      Retrieves the Application Name of the currently used vic application 
#                   In version 2.7 the name is "vic"   
# Add Info:         This information is also included in $V(app)
# 
# 
# ***************************************************************************
# 
proc Vic.GetApplicationName { } {
    return vic
}






# ***************************************************************************
#  
# Procedure:        Vic.GetDataNet
# Date:             05/06/96
# Param:            -
# Result:           $DataNet
# Description:      Retrieves the DataNet handle of the currently used vic application 
#                   This information is delivered by $V(data-net)
#                   and can be used to retrieve subsequently address, port and ttl inf. 
#                  	
#  
# ***************************************************************************
#  
proc Vic.GetDataNet { } {

	set App [Vic.GetApplicationName]
	set Cmd "set V(data-net)"
	catch [list send $App $Cmd] DataNet
	return $DataNet

}




# ***************************************************************************
#
# Procedure:        Vic.GetMyAddress
# Date:             05/06/96
# Param:            -
# Result:           $MyAddress
# Description:      Retrieves the internet address of the currently used vic application
#                   This information is delivered by "$V(data-net) addr"
#                   E.g. _o3 addr
#
#
# ***************************************************************************
#
proc Vic.GetMyAddress { } {
 
    set App [Vic.GetApplicationName]
	set DataNet [Vic.GetDataNet]
    set Cmd "$DataNet addr"
    catch [list send $App $Cmd] MyAddress
    return $MyAddress
 
}




# ***************************************************************************
#
# Procedure:        Vic.GetMyPort
# Date:             05/06/96
# Param:            -
# Result:           $MyPort
# Description:      Retrieves the internet port of the currently used vic application
#                   This information is delivered by "$V(data-net) port"
#                   E.g. _o3 port
#
#
# ***************************************************************************
#
proc Vic.GetMyPort { } {
 
    set App [Vic.GetApplicationName]
    set DataNet [Vic.GetDataNet]
    set Cmd "$DataNet port"
    catch [list send $App $Cmd] MyPort  
    return $MyPort
 
}





# ***************************************************************************
#
# Procedure:        Vic.GetMyTTL
# Date:             05/06/96
# Param:            -
# Result:           $MyTTL
# Description:      Retrieves the TTL value of the currently used vic application
#                   This information is delivered by "$V(data-net) ttl"
#                   E.g. _o3 addr
#
#
# ***************************************************************************
#
proc Vic.GetMyTTL { } {
 
    set App [Vic.GetApplicationName]
    set DataNet [Vic.GetDataNet]
    set Cmd "$DataNet ttl"
    catch [list send $App $Cmd] MyTTL  
    return $MyTTL
 
}








# ***************************************************************************
#
# Procedure:        Vic.SetCtrlMenu
# Date:             05/07/96
# Param:            -
# Result:           -
# Description:      Retrieves the status of the vic control menu (defined/undefined)
#                   If undefined (i.e. window doesn't exist) the procedure
#                   "toggle_window .menu" in "ui-ctrlmnu.tcl" is invoked twice (open/close)
#
#
# ***************************************************************************
#
proc Vic.SetCtrlMenu { } {
 
    set App [Vic.GetApplicationName]
    set Cmd "winfo exists .menu"
    catch [list send $App $Cmd] WindowExists

	if {$WindowExists == 0} {
		set Cmd "toggle_window .menu"
		catch [list send $App $Cmd]
		catch [list send $App $Cmd]
	} 
}






# ***************************************************************************
#
# Procedure:        Vic.GetTransmitStatus
# Date:             05/07/96
# Param:            -
# Result:           $MyTransmitStatus
# Description:      Retrieves the transmit statusof the currently used vic application
#                   This information is delivered by the variable "transmitButtonState" 
#                   E.g. transmitButtonState = 1(on); transmitButtonState =0 (off)
#
#
# ***************************************************************************
#
proc Vic.GetTransmitStatus { } {
	
	Vic.SetCtrlMenu 
    set App [Vic.GetApplicationName]
    set Cmd "set transmitButtonState"
    catch [list send $App $Cmd] MyTransmitStatus
    return $MyTransmitStatus

}     





     
# ***************************************************************************
#
# Procedure:        Vic.SetTransmit
# Date:             05/07/96
# Param:            -
# Result:           1: Had to invoke transmit button ; 0: no action taken
# Description:      Retrieves the transmit status via Vic.GetTransmitStatus
#                   and invokes subsequently the transmit button
#                   E.g. 
#
#
# ***************************************************************************
#
proc Vic.SetTransmit { } {

	if { [Vic.GetTransmitStatus] == 0 } {
		set App [Vic.GetApplicationName]
		set Cmd ".menu.cb.frame.buttons.send invoke"
		catch [list send $App $Cmd] 
		return 1
	} else {
		return 0
	}
}







# ***************************************************************************
#
# Procedure:        Vic.SetRelease
# Date:             05/07/96
# Param:            -
# Result:           
# Description:      Invokes the release button
#                   E.g. 
#
#
# ***************************************************************************
#
proc Vic.SetRelease { } {
	
	if { [Vic.GetTransmitStatus] != 0 } {
		set App [Vic.GetApplicationName]
		set Cmd ".menu.cb.frame.buttons.release invoke"
		catch [list send $App $Cmd] 
	}
}




# ***************************************************************************
#
# Procedure:        Vic.GetMyKBPS
# Date:             05/07/96
# Param:            -
# Result:           $MyKBPS
# Description:      Retrieves my sending rate in kbps from the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   E.g. 
#
#
# ***************************************************************************
#
proc Vic.GetMyKBPS { } {
   
	set App [Vic.GetApplicationName]
	set Cmd ".menu.cb.frame.right.info.bps cget -text"
	catch [list send $App $Cmd] MyKBPS
	return $MyKBPS
}

	 



# ***************************************************************************
#
# Procedure:        Vic.GetMyFPS
# Date:             05/07/96
# Param:            -
# Result:           $MyFPS
# Description:      Retrieves my frame sending rate in fps from the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   E.g. 
#
#
# ***************************************************************************
#
proc Vic.GetMyFPS { } {
   
    set App [Vic.GetApplicationName]
    set Cmd ".menu.cb.frame.right.info.fps cget -text"
    catch [list send $App $Cmd] MyFPS
    return $MyFPS
}






# ***************************************************************************
#
# Procedure:        Vic.GetMyMaxKBPS
# Date:             05/07/96
# Param:            -
# Result:           $MyMaxKBPS
# Description:      Retrieves my maximum sending rate in kbps from the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   determined by the slider value under Rate Control 
#
#
# ***************************************************************************
#
proc Vic.GetMyMaxKBPS { } {
   
    set App [Vic.GetApplicationName]
    set Cmd ".menu.cb.frame.right.bps.value cget -text"
    catch [list send $App $Cmd] MyMaxKBPS
    return $MyMaxKBPS
}






# *************************************************************************** 
# 
# Procedure:        Vic.GetMyMaxFPS 
# Date:             05/07/96   
# Param:            - 
# Result:           $MyMaxFPS 
# Description:      Retrieves my maximum frame sending rate in fps from the 
#                   vic control panel (--> ui-ctrlmenu.tcl) 
#                   determined by the slider value under Rate Control  
# 
# 
# *************************************************************************** 
# 
proc Vic.GetMyMaxFPS { } { 
    
    set App [Vic.GetApplicationName] 
    set Cmd ".menu.cb.frame.right.fps.value cget -text" 
    catch [list send $App $Cmd] MyMaxFPS    
    return $MyMaxFPS 
} 





# ***************************************************************************
#
# Procedure:        Vic.SetMyMaxKBPS
# Date:             05/07/96
# Param:            $KBPS: Kilobits per second (10..3072) 
# Result:           -
# Description:      Sets my maximum sending rate in kbps from the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   determined by the slider scale under Rate Control
#
#
# ***************************************************************************
#
proc Vic.SetMyMaxKBPS { KBPS } {
    
    set App [Vic.GetApplicationName]
    set Cmd ".menu.cb.frame.right.bps.scale set $KBPS"
    catch [list send $App $Cmd] 
}




# ***************************************************************************
#
# Procedure:        Vic.SetMyMaxFPS
# Date:             05/07/96
# Param:            $FPS: Frames per second (1..30)
# Result:           -
# Description:      Sets my maximum frame sending rate in fps from the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   determined by the slider scale under Rate Control
#
#
# ***************************************************************************
#
proc Vic.SetMyMaxFPS { FPS } {
 
    set App [Vic.GetApplicationName]
    set Cmd ".menu.cb.frame.right.fps.scale set $FPS"
    catch [list send $App $Cmd]
}








# ***************************************************************************
#
# Procedure:        Vic.GetQuality
# Date:             05/07/96
# Param:            - 
# Result:          	$Quality (0 .. 99) 
# Description:      Gets my quality value in the encoding area in the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   determined by the corresponding scale 
#
#
# ***************************************************************************
#
proc Vic.GetQuality { } {
 
    set App [Vic.GetApplicationName]
    set Cmd ".menu.encoder.f.h1.tb.scale get" 
    catch [list send $App $Cmd] Quality
	return $Quality
}





# ***************************************************************************
#
# Procedure:        Vic.GetQualityEncoder
# Date:             05/07/96
# Param:            -
# Result:           $QualityEncoder (x  .. y) 
# Description:      Gets my quality value in the encoding area in the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   determined by the corresponding label
#					The value depends on the selected encoding scheme
#
#
# ***************************************************************************
#
proc Vic.GetQualityEncoder { } {
 
    set App [Vic.GetApplicationName]
    set Cmd ".menu.encoder.f.h1.tb.value cget -text" 
    catch [list send $App $Cmd] QualitEncoder
    return $QualitEncoder
}



# ***************************************************************************
#
# Procedure:        Vic.SetQuality
# Date:             05/07/96
# Param:            $Quality (0..99)
# Result:           -
# Description:      Sets my quality value in the encoding area in the
#                   vic control panel (--> ui-ctrlmenu.tcl)
#                   determined by the corresponding scale
#
#
# ***************************************************************************
#
proc Vic.SetQuality { Quality } {
 
    set App [Vic.GetApplicationName]
    set Cmd ".menu.encoder.f.h1.tb.scale set $Quality " 
    catch [list send $App $Cmd] 
}






# ***************************************************************************
#
# Procedure:        Vic.GetEncoding
# Date:             05/07/96
# Param:            -
# Result:           $EncodingScheme: E.g. nv, jpeg, h261...
# Description:      Retrieves the encoding scheme of the active
#                   vic application (Tcl Global Variable: videoFormat)
#
#
# ***************************************************************************
#
proc Vic.GetEncoding { } {
 
    set App [Vic.GetApplicationName]
    set Cmd "set videoFormat"
    catch [list send $App $Cmd] EncodingScheme
	return $EncodingScheme
}
 







# ***************************************************************************
#
# Procedure:        Vic.SetEncoding
# Date:             05/07/96
# Param:            $EncodingScheme (nv, nvdct,cellb,jpeg,h261,bvc)
# Result:           
# Description:      Sets the encoding scheme of the active
#                   vic application 
#
#
# ***************************************************************************
#
proc Vic.SetEncoding { EncodingScheme } {
 
    set App [Vic.GetApplicationName]
	if { $EncodingScheme == "nv" } {
		set column p0
		set row b0
	} elseif {$EncodingScheme == "nvdct" } {
        set column p0 
        set row b1   
    } elseif {$EncodingScheme == "cellb" } { 
        set column p0  
        set row b2
    } elseif {$EncodingScheme == "jpeg" } { 
        set column p1  
        set row b0
    } elseif {$EncodingScheme == "h261" } { 
        set column p1  
        set row b1
    } elseif {$EncodingScheme == "bvc" } { 
        set column p1  
        set row b2
	}
    set Cmd ".menu.encoder.f.h0.format.$column.$row invoke"
    catch [list send $App $Cmd] 
}







# ***************************************************************************
#
# Procedure:        Vic.ShowWindow
# Date:             05/08/96
# Param:            $Window : Windowname
# Result:
# Description:      Displays the vic window $Window
#                   E.g. vic top window (.)
#
#
# ***************************************************************************
#
proc Vic.ShowWindow { Window } {
 
    set App [Vic.GetApplicationName]
    set Cmd "wm deiconify $Window"
    catch [list send $App $Cmd]
}




# ***************************************************************************
#
# Procedure:        Vic.HideWindow 
# Date:             05/08/96 
# Param:            $Window : Windowname
# Result: 
# Description:      Hides the vic window $Window
#                   E.g. vic top window (.)
# 
# 
# *************************************************************************** 
# 
proc Vic.HideWindow { Window } {
  
    set App [Vic.GetApplicationName]
    set Cmd "wm withdraw $Window"
    catch [list send $App $Cmd] 
} 








# ***************************************************************************
#
# Procedure:        Vic.FindHandle
# Date:             05/08/96
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
# Result:			$Handle (e.g. _o6) ; If no Handle available return 0
# Description:      Searches for the corresponding vic handle/object (e.g. _o6)
#                   to a given participant
#
#
# ***************************************************************************
#
proc Vic.FindHandle { Participant } {

    set App [Vic.GetApplicationName]

	# 1. Scan the "active array elements" --> IndexList
	set Cmd "array names active"

	set Handle 0
	foreach i [ send $App $Cmd ] {
		set Cmd ".$i.r.cw.addr cget -text"
		catch [list send $App $Cmd] NameInternet

		# extract EncodingScheme
		set Separator [string first / $NameInternet]
		set NameInternet [string range $NameInternet 0 [expr $Separator-1] ]
		
		# Compare NameInternet with Participant
		if { [string compare $NameInternet $Participant]== 0 } {
			set Handle $i
		}		
	}
	return $Handle
}







# ***************************************************************************
#
# Procedure:        Vic.GetInfo
# Date:             05/08/96
# Last changed:		06/25/96 (added error case)
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
#					$Info: Type of information, e.g. (Name, FPS, KBPS, Loss, 
#							EncodingScheme, Color
# Result:           $Value of $Info attribute; 0 if error
# Description:      Retrieves the value of the ordered information type
#                   of a given participant
#
#
# ***************************************************************************
#
proc Vic.GetInfo { Participant Info } {
 
    set App [Vic.GetApplicationName]
	set Handle [Vic.FindHandle $Participant] 

	if { $Handle == 0 } {
		return 0
	}

	if { $Info == "Name" } {
		set Cmd	".$Handle.r.cw.name cget -text "
        catch [list send $App $Cmd] Value
	} elseif { $Info == "FPS"} {
		set Cmd "set ftext($Handle)"
        catch [list send $App $Cmd] Value
	} elseif { $Info == "KBPS"} {
		set Cmd "set btext($Handle)"
        catch [list send $App $Cmd] Value
	} elseif { $Info == "Loss"} {
		set Cmd "set ltext($Handle)"
        catch [list send $App $Cmd] Value
	} elseif { $Info == "EncodingScheme"} {
		set Cmd ".$Handle.r.cw.addr cget -text"
        catch [list send $App $Cmd] Address
		set Separator [string first / $Address]
		set EncodingScheme [string range $Address [expr $Separator+1] end]
		set Value $EncodingScheme
	} elseif { $Info == "Color"} {
		set Cmd "set colorbutton($Handle)"
        catch [list send $App $Cmd] Value
	}

    return $Value
}






# ***************************************************************************
#
# Procedure:        Vic.GetRTPInfo
# Date:             05/08/96
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
#                   
# Result:           $RTPInfo (containing Kilobits, Frames, Packets, Loss, ...) 
# Description:      Retrieves a string comprising all the RTP Info at a 
#                   certain time
#
#
# ***************************************************************************
#
proc Vic.GetRTPInfo { Participant } {
 
    set App [Vic.GetApplicationName]    
    set Handle [Vic.FindHandle $Participant]


    set Cmd "$Handle stats"
    catch [list send $App $Cmd] RTPInfo
	
 
    return $RTPInfo
}








# ***************************************************************************
#
# Procedure:        Vic.DisplayParticipant
# Date:             05/09/96
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
#
# Result:           
# Description:      Displays the window of a given participant
#                   through the select_thumbnail Vic Procedure
# Syntax:			"select_thumbnail <window> <handle>		"
#
#
# ***************************************************************************
#
proc Vic.DisplayParticipant { Participant } {
 
    set App [Vic.GetApplicationName]
    set Handle [Vic.FindHandle $Participant]
 
    set Cmd "select_thumbnail .$Handle $Handle"
    catch [list send $App $Cmd] 
   
}


# ***************************************************************************
#
# Procedure:        Vic.Quit
# Date:             05/30/96
# Param:            -
# Result:           -
# Description:      Quit Vic Application
#                   
# ***************************************************************************
#
proc Vic.Quit { } {
 
    set App [Vic.GetApplicationName]
    set Cmd "adios"
    catch [list send $App $Cmd] 
 
}


# ***************************************************************************
#
# Procedure:        Vic.GetParticipantList
# Date:             06/26/96
# Param:            -
# Result:    		$ParticipantList (list of active participants)       
# Description:      Quit Vic Application
#
# ***************************************************************************
#
proc Vic.GetParticipantList { } {

    set App [Vic.GetApplicationName]
	set ParticipantList ""

    # 1. Scan the "active array elements" --> IndexList
    set Cmd "array names active"

    set Handle ""
    foreach i [ send $App $Cmd ] {
        set Cmd ".$i.r.cw.addr cget -text"
        catch [list send $App $Cmd] NameInternet

        # extract EncodingScheme
        set Separator [string first / $NameInternet]
        set NameInternet [string range $NameInternet 0 [expr $Separator-1] ]
        
        # append Name to list $ParticopantList, Separator is "|"
		append ParticipantList $NameInternet
		append ParticipantList "|"
    }
    return $ParticipantList
}


# ***************************************************************************
#
# Procedure:        Vic.IsAlive
# Date:             07/24/96
# Param:            -
# Result:           $Alive (1 = is alive ) ( 0 = is not alive )
# Description:      Check if vic application is alive
#
# ***************************************************************************
#
proc Vic.IsAlive { } {

	set App [Vic.GetApplicationName]
	if { [lsearch [winfo interps] $App] != -1 } {
		# Vic is alive
		set Alive 1
	} else {
		# Vic is dead
		set Alive 0
	}

	return $Alive
}

	

